import { world, system } from "@minecraft/server";
import { ActionFormData } from "@minecraft/server-ui";

// Price configuration per item group
const ITEM_GROUPS = {
    Logs: {
        items: ["minecraft:oak_log"],
        price: 3
    },
    Coal: {
        items: ["minecraft:coal"],
        price: 5
    },
    Iron: {
        items: ["minecraft:iron_ingot"],
        price: 15
    },
    Gold: {
        items: ["minecraft:gold_ingot", "minecraft:gold_block"],
        price: { "minecraft:gold_ingot": 20, "minecraft:gold_block": 200 }
    },
    Diamonds: {
        items: ["minecraft:diamond", "minecraft:diamond_block"],
        price: { "minecraft:diamond": 20, "minecraft:diamond_block": 180 }
    },
    Netherite: {
        items: ["minecraft:netherite_ingot", "minecraft:netherite_block"],
        price: { "minecraft:netherite_ingot": 115, "minecraft:netherite_block": 1035 }
    },
    CryingObsidian: {
        items: ["minecraft:crying_obsidian"],
        price: 5000
    },
    Amethyst: {
        items: ["minecraft:amethyst_shard"],
        price: 3
    },
    Emeralds: {
        items: ["minecraft:emerald", "minecraft:emerald_block"],
        price: { "minecraft:emerald": 12, "minecraft:emerald_block": 150 }
    },
    Lapis: {
        items: ["minecraft:lapis_lazuli"],
        price: 3
    },
    Redstone: {
        items: ["minecraft:redstone"],
        price: 3
    },
    Copper: {
        items: ["minecraft:copper_ingot"],
        price: 12
    },
    Magma: {
        items: ["minecraft:magma"],
        price: 8000
    }
};

// Per-player settings stored as tag keys like "autosell:Diamonds:on"
function getSetting(player, group) {
    return player.getTags().includes(`autosell:${group}:on`);
}

function toggleSetting(player, group) {
    const on = getSetting(player, group);
    player.removeTag(`autosell:${group}:${on ? "on" : "off"}`);
    player.addTag(`autosell:${group}:${on ? "off" : "on"}`);
}

export default async function AutoSellMenu(player) {
    if (!player.getTags().includes("autosell")) {
        player.sendMessage("§cYou do not have permission to use AutoSell.");
        return;
    }

    const groups = Object.keys(ITEM_GROUPS);
    const form = new ActionFormData()
        .title("§6AutoSell Menu")
        .body("§7Toggle which blocks should be sold automatically");

    // Add "Toggle All" button
    form.button("§eToggle All");

    for (const group of groups) {
        const enabled = getSetting(player, group);
        form.button(`${enabled ? "§a✔" : "§c✘"} §f${group}`);
    }

    form.button("§cExit");

    const { canceled, selection } = await form.show(player);
    if (canceled || selection === groups.length + 1) return;

    if (selection === 0) {
        // Toggle all groups
        const someDisabled = groups.some(g => !getSetting(player, g));
        for (const group of groups) {
            player.removeTag(`autosell:${group}:on`);
            player.removeTag(`autosell:${group}:off`);
            player.addTag(`autosell:${group}:${someDisabled ? "on" : "off"}`);
        }
    } else {
        const selectedGroup = groups[selection - 1]; // adjust for Toggle All
        toggleSetting(player, selectedGroup);
    }

    AutoSellMenu(player); // re-open menu to reflect changes
}

// Auto-sell runner
system.runInterval(() => {
    for (const player of world.getPlayers()) {
        if (!player.getTags().includes("autosell")) continue;
        const inventory = player.getComponent("inventory").container;
        if (!inventory) continue;

        const multiplier = player.getTags().some(t => t === "2xsell" || t === "pre") ? 2 : 1;

        for (const [group, data] of Object.entries(ITEM_GROUPS)) {
            if (!getSetting(player, group)) continue;

            for (let i = 0; i < inventory.size; i++) {
                const item = inventory.getItem(i);
                if (!item) continue;

                const type = item.typeId;
                if (!data.items.includes(type)) continue;

                const price = typeof data.price === "object" ? data.price[type] : data.price;
                const totalEarned = price * item.amount * multiplier;

                // Remove and credit
                inventory.setItem(i, undefined);
                addMoney(player, totalEarned);
                player.sendMessage(`§aAutoSold §f${item.amount}x ${type.replace("minecraft:", "")} §afor §6$${totalEarned}`);
            }
        }
    }
}, 100); // every ~5 seconds

function addMoney(player, amount) {
    try {
        const objective = world.scoreboard.getObjective("money");
        const score = objective.getScore(player.scoreboardIdentity);
        objective.setScore(player.scoreboardIdentity, score + amount);
    } catch {
        // silently fail
    }
}
