import { system, world } from "@minecraft/server";
import { ModalFormData } from "@minecraft/server-ui";

const MAX_BALANCE = 100000000000; // 10 billion
const MAX_SCOREBOARD = 2000000000; // 2 billion

function getBankBalanceFromTag(player) {
  const tag = player.getTags().find(t => t.startsWith("bank_"));
  return tag ? parseInt(tag.substring(5)) : 0;
}

function setBankBalanceTag(player, amount) {
  // Remove old bank_ tags
  player.getTags().forEach(tag => {
    if (tag.startsWith("bank_")) {
      player.removeTag(tag);
    }
  });
  player.addTag(`bank_${amount}`);
}

function migrateScoreboardBankToTag(player, bankObj, identity) {
  const scoreboardBank = bankObj.getScore(identity);
  if (typeof scoreboardBank === "number") {
    setBankBalanceTag(player, scoreboardBank);
    bankObj.setScore(identity, 0); // Clear old value
    return scoreboardBank;
  }
  return 0;
}

export default function Bank(player) {
  const identity = player.scoreboardIdentity;
  const moneyObj = world.scoreboard.getObjective("money");
  const bankObj = world.scoreboard.getObjective("bank");

  if (!identity || !moneyObj || !bankObj) {
    player.sendMessage("§cScoreboard objectives are missing or not set up.");
    return;
  }

  const money = moneyObj.getScore(identity) ?? 0;

  // --- Handle tag-based bank balance ---
  let bank = getBankBalanceFromTag(player);
  if (bank === 0) {
    // Attempt migration if no tag exists
    bank = migrateScoreboardBankToTag(player, bankObj, identity);
  }

  const balanceInfo = `§7Your Balances:\n§7Money: §6${money.toLocaleString()}\n§7Bank: §b${bank.toLocaleString()}`;

  new ModalFormData()
    .title("§aBank")
    .dropdown("§7What do you want to do?", ["Deposit", "Withdraw"])
    .textField(`${balanceInfo}\n\n§7Enter amount:`, "100")
    .show(player)
    .then(({ canceled, formValues }) => {
      if (canceled) return;
      const [action, amountStr] = formValues;
      const amount = parseInt(amountStr);

      if (isNaN(amount) || amount <= 0) {
        player.sendMessage("§cInvalid amount entered.");
        return;
      }

      if (action === 0) { // Deposit
        if (money < amount)
          return player.sendMessage("§cYou don't have enough money to deposit.");
        if (bank + amount > MAX_BALANCE)
          return player.sendMessage("§cYou can have a maximum of 100,000,000,000$ in the bank. Please deposit less.");

        moneyObj.setScore(identity, money - amount);
        bank += amount;
        setBankBalanceTag(player, bank);
        player.sendMessage(`§aDeposited §e${amount.toLocaleString()}§a to your bank.`);

      } else { // Withdraw
        if (bank < amount)
          return player.sendMessage("§cYou don't have that much in your bank.");
        if (money + amount > MAX_SCOREBOARD)
          return player.sendMessage("§cCannot withdraw: money would exceed 2,000,000,000$ limit.");

        moneyObj.setScore(identity, money + amount);
        bank -= amount;
        setBankBalanceTag(player, bank);
        player.sendMessage(`§aWithdrew §e${amount.toLocaleString()}§a from your bank.`);
      }
    });
}
