import { world, system } from "@minecraft/server";
import { ModalFormData, MessageFormData } from "@minecraft/server-ui";

const pendingTPARequests = new Map();

/**
 * TPA command: sends teleport request to another player.
 * @param {import("@minecraft/server").Player} requester
 */
export default function TPA(requester) {
    const players = world.getPlayers().filter(p =>
        p.name !== requester.name &&
        !p.getTags().includes(`blocktpa_${requester.name}`) &&
        !p.getTags().includes("blocktpa_all")
    );

    if (players.length === 0) {
        requester.sendMessage("§cNo available players to send a request to.");
        return;
    }

    const form = new ModalFormData()
        .title("§9TPA Request")
        .dropdown("§7Select a player to send a teleport request:", players.map(p => p.name));

    form.show(requester).then(({ canceled, formValues }) => {
        if (canceled) return;

        const target = players[formValues[0]];

        // Final check
        if (!target || target.getTags().includes(`blocktpa_${requester.name}`) || target.getTags().includes("blocktpa_all")) {
            requester.sendMessage(`§c${target?.name ?? "Player"} is no longer accepting TPA requests.`);
            return;
        }

        // Save pending request
        const timeoutId = system.runTimeout(() => {
            pendingTPARequests.delete(target.name);
            requester.sendMessage(`§eTeleport request to §f${target.name} §ehas expired.`);
        }, 20 * 15); // 15 seconds

        pendingTPARequests.set(target.name, {
            from: requester,
            timeoutId
        });

        // Send request to target
        const messageForm = new MessageFormData()
            .title("§dTeleport Request")
            .body(`§b${requester.name} §7wants to teleport to you.`)
            .button1("§aAccept")             // selection === 0
            .button2("§cDeny");              // selection === 1

        const extraOptionsForm = new MessageFormData()
            .title("§4Block Teleport Requests?")
            .body(`§cWould you like to block all future teleport requests from §f${requester.name}§c?`)
            .button1("§4Deny & Block")       // selection === 0
            .button2("§7Just Deny");         // selection === 1

        messageForm.show(target).then(({ canceled, selection }) => {
            if (!pendingTPARequests.has(target.name)) return;

            const { from, timeoutId } = pendingTPARequests.get(target.name);
            pendingTPARequests.delete(target.name);
            system.clearRun(timeoutId);

            if (canceled) {
                target.sendMessage("§eTeleport request ignored.");
                from.sendMessage(`§e${target.name} did not respond.`);
                return;
            }

            if (selection === 0) {
                const loc = target.location;
                from.teleport({ x: loc.x, y: loc.y + 1, z: loc.z });
                from.sendMessage(`§aYou have been teleported to §e${target.name}`);
                target.sendMessage(`§aYou accepted the teleport request from §e${from.name}`);
            } else {
                extraOptionsForm.show(target).then(({ canceled, selection }) => {
                    if (canceled || selection === 1) {
                        target.sendMessage(`§cYou denied the teleport request from §e${from.name}`);
                        from.sendMessage(`§c${target.name} denied your teleport request.`);
                    } else if (selection === 0) {
                        target.addTag(`blocktpa_${from.name}`);
                        target.sendMessage(`§cYou denied and blocked future requests from §e${from.name}`);
                        from.sendMessage(`§c${target.name} denied and blocked your teleport request.`);
                    }
                });
            }
        });

        requester.sendMessage(`§7Teleport request sent to §e${target.name}`);
    });
}
